Here is the data:

Input:
'''
{{ input_text }}
'''
{% if contexts_text %}

References:
'''
{{ contexts_text }}
'''
{% endif %}
{% if history_messages_text %}

Conversation:
'''
{{ history_messages_text }}
'''
{% endif %}

Output:
'''
{{ output_text }}
'''

Answer as JSON:
